# Michael Halle
# mhalle@bwh.harvard.edu
# version 0.2.0

__doc__="""Block: define a block in a template.
     Arguments to dtml-block are just like those to dtml-var, except
     that only names (not expressions) can be specified.  If the block's
     name is available in the current namespace, the object with that
     name is inserted.  If no such object exists, the text in the block
     is inserted.

     If 'nosubst' is set and not equal to 'false', a variable value is never
     substituted.  If 'private' is set and not equal to 'false', the
     block is not available to other documents.

     """

from DocumentTemplate.DT_String import String
from DocumentTemplate.DT_Var    import Var, modifiers
from DocumentTemplate.DT_Util import render_blocks
import DocumentTemplate.DT_Util

falsevals = ("false", "FALSE", "0", 0)

class Block(Var):
    name='block'
    blockContinuations=()

    def __init__(self, blocks, fmt='s'):
        tname, args, section = blocks[0]
        self.section = section.blocks

        args = self.args = parse_params(args)
        name, expr = name_param(args)
        if not name:
            raise 'Document Error', ("All blocks must be named")

        self.__name__, self.expr = name, expr

        self.modifiers=tuple(
            map(lambda t: t[1],
                filter(lambda m, args=args, used=args.has_key:
                       used(m[0]) and args[m[0]],
                       modifiers)))

        self.fmt = fmt

    def render(self, md):  
        name=self.__name__

        import sys
        d = {}
        if self.args.has_key('nosubst') or not md.has_key(name):

            # Normally, in Var, we'd blow out right here,
            # because the variable name isn't bound.
            # Instead, we'll substitute in our block
            # contents.

            # If nosubst is set, never substitute.
            
            d[name] = render_blocks(self.section, md)
            
        md._push(d)
        s = Var.render(self, md)
        md._pop(1)
        return s

    __call__=render


def parse_params(args):
    args = DocumentTemplate.DT_Util.parse_params(
        args, name='', lower=1, upper=1, 
        capitalize=1, spacify=1, null='', fmt='s',
        size=0, etc='...', thousands_commas=1,
        html_quote=1, url_quote=1, sql_quote=1,
        url_quote_plus=1, missing='',
        newline_to_br=1, url=1,
        nosubst=1, private=1)

    # 'nosubst' means don't substitute even if the variable is set
    # in the environment.
        
    k = 'nosubst'
    if args.has_key(k):
        if args[k] in falsevals:
            del(args[k])

    # 'private' means don't make your blocks available to other documents.
    k = 'private'
    if args.has_key(k):
        if args[k] in falsevals:
            del(args[k])
        elif not args.has_key('nosubst'):
            raise 'Document Error', (
                "Use of the 'private' attribute without 'nosubst' currently unsupported")

    return args


def name_param(argd):
    return DocumentTemplate.DT_Util.name_param(argd, Block.name, 1)

String.commands[Block.name] = Block

class Suppress:
    '''Suppress

    The 'suppress' suppresses the output of any DTML source
    contained within it.  It is functionally equivalent to
    the comment tag.
    
    For example::
    
      <dtml-suppress>
      
        This text is not rendered.

      </dtml-suppress>
    ''' 
    name='suppress'
    blockContinuations=()

    def __init__(self, args, fmt=''): pass

    def render(self, md):
        return ''

    __call__=render

String.commands[Suppress.name] = Suppress

